@@ -2,16 +2,22 @@
 import futures
 
 import os
+import json
 import socket
+
 from collections import OrderedDict
-from distutils.version import LooseVersion
 try:
-    from xmlrpclib import ServerProxy
+    from urllib2 import urlopen
+    from urllib2 import URLError
     from ConfigParser import NoSectionError
 except ImportError:  # Python 3
-    from xmlrpc.client import ServerProxy
+    from urllib.error import URLError
+    from urllib.request import urlopen
     from configparser import NoSectionError
 
+from packaging.specifiers import SpecifierSet
+from packaging.version import parse as parse_version
+
 from bvc.logger import logger
 from bvc.configparser import VersionsConfigParser
 
@@ -22,7 +28,9 @@ class VersionsChecker(object):
     """
     default_version = '0.0.0'
 
-    def __init__(self, source, includes=[], excludes=[],
+    def __init__(self, source,
+                 specifiers={}, allow_pre_releases=False,
+                 includes=[], excludes=[],
                  service_url='http://pypi.python.org/pypi',
                  timeout=10, threads=10):
         """
@@ -32,15 +40,21 @@ class VersionsChecker(object):
         self.source = source
         self.includes = includes
         self.excludes = excludes
+        self.specifiers = specifiers
+        self.allow_pre_releases = allow_pre_releases
         self.timeout = timeout
         self.threads = threads
         self.service_url = service_url
+
         self.source_versions = OrderedDict(
             self.parse_versions(self.source))
         self.versions = self.include_exclude_versions(
             self.source_versions, self.includes, self.excludes)
+        self.package_specifiers = self.build_specifiers(
+            self.versions.keys(), self.specifiers)
         self.last_versions = OrderedDict(
-            self.fetch_last_versions(self.versions.keys(),
+            self.fetch_last_versions(self.package_specifiers,
+                                     self.allow_pre_releases,
                                      self.service_url,
                                      self.timeout,
                                      self.threads))
@@ -81,9 +95,22 @@ class VersionsChecker(object):
                     len(versions))
         return versions
 
-    def fetch_last_versions(self, packages, service_url, timeout, threads):
+    def build_specifiers(self, packages, source_specifiers):
         """
-        Fetch the latest versions of a list of packages,
+        Builds a list of tuple (package, version specifier)
+        """
+        specifiers = []
+        source_specifiers = dict((k.lower(), v) for k, v in
+                                 source_specifiers.items())
+        for package in packages:
+            specifier = source_specifiers.get(package.lower(), '')
+            specifiers.append((package, specifier))
+        return specifiers
+
+    def fetch_last_versions(self, packages, allow_pre_releases,
+                            service_url, timeout, threads):
+        """
+        Fetch the latest versions of a list of packages with specifiers,
         in a threaded manner or not.
         """
         versions = []
@@ -91,33 +118,42 @@ class VersionsChecker(object):
             with futures.ThreadPoolExecutor(
                     max_workers=threads) as executor:
                 tasks = [executor.submit(self.fetch_last_version,
-                                         package, service_url, timeout)
+                                         package, allow_pre_releases,
+                                         service_url, timeout)
                          for package in packages]
                 for task in futures.as_completed(tasks):
                     versions.append(task.result())
         else:
             for package in packages:
                 versions.append(self.fetch_last_version(
-                    package, service_url, timeout))
+                    package, allow_pre_releases, service_url, timeout))
         return versions
 
-    def fetch_last_version(self, package, service_url, timeout):
+    def fetch_last_version(self, package, allow_pre_releases,
+                           service_url, timeout):
         """
         Fetch the last version of a package on Pypi.
         """
-        package_key = package.lower()
-        max_version = self.default_version
+        package, specifier = package
+        specifier = SpecifierSet(specifier, allow_pre_releases)
+        max_version = parse_version(self.default_version)
         logger.info('> Fetching latest datas for %s...' % package)
+        package_json_url = '%s/%s/json' % (service_url, package)
         socket.setdefaulttimeout(timeout)
-        client = ServerProxy(service_url)
-        results = client.search({'name': package})
+        try:
+            content = urlopen(package_json_url).read()
+        except URLError as error:
+            content = '{"releases": []}'
+            logger.debug('!> %s %s' % (package_json_url, error.reason))
+        results = json.loads(content)
         socket.setdefaulttimeout(None)
-        for result in results:
-            if result['name'].lower() == package_key:
-                if LooseVersion(result['version']) > LooseVersion(max_version):
-                    max_version = result['version']
-        logger.debug('-> Last version of %s is %s.' % (package, max_version))
-        return (package, max_version)
+        for version in specifier.filter(results['releases']):
+            version = parse_version(version)
+            if version > max_version:
+                max_version = version
+        logger.debug('-> Last version of %s%s is %s.' % (
+            package, specifier, max_version))
+        return (package, str(max_version))
 
     def find_updates(self, versions, last_versions):
         """
