@@ -2,55 +2,90 @@
 from six import string_types
 
 import sys
+import copy
 import logging
+from argparse import Action
+from argparse import ArgumentError
 from argparse import ArgumentParser
+from argparse import _ensure_value
 
 from bvc.logger import logger
 from bvc.checker import VersionsChecker
 from bvc.configparser import VersionsConfigParser
 
 
+class StoreSpecifiers(Action):
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        items = copy.copy(_ensure_value(namespace, self.dest, {}))
+        try:
+            key, value = values.split(':')
+        except ValueError:
+            raise ArgumentError(self, 'key:value syntax not followed')
+        key = key.strip()
+        value = value.strip()
+        if not key or not value:
+            raise ArgumentError(self, 'key or value are empty')
+        items.update({key: value})
+        setattr(namespace, self.dest, items)
+
+
 def cmdline(argv=sys.argv[1:]):
     parser = ArgumentParser(
         description='Check availables updates from a '
         'version section of a buildout script')
     parser.add_argument(
-        '-s', '--source', dest='source', default='versions.cfg',
+        'source', default='versions.cfg', nargs='?',
         help='The file where versions are pinned '
         '(default: versions.cfg)')
-    parser.add_argument(
+    version_group = parser.add_argument_group('Allowed versions')
+    version_group.add_argument(
+        '--pre', action='store_true', dest='prereleases', default=False,
+        help='Allow pre-releases and development versions '
+        '(by default only stable versions are found)')
+    version_group.add_argument(
+        '-s', '--specifier', action=StoreSpecifiers,
+        dest='specifiers', default={},
+        help='Describe what versions of a package are acceptable. '
+        'Example "package:>=1.0,!=1.3.4.*,< 2.0" '
+        '(can be used multiple times)')
+    filter_group = parser.add_argument_group('Filtering')
+    filter_group.add_argument(
         '-i', '--include', action='append', dest='includes', default=[],
         help='Include package when checking updates '
         '(can be used multiple times)')
-    parser.add_argument(
+    filter_group.add_argument(
         '-e', '--exclude', action='append', dest='excludes', default=[],
         help='Exclude package when checking updates '
         '(can be used multiple times)')
-    parser.add_argument(
+    file_group = parser.add_argument_group('File')
+    file_group.add_argument(
         '-w', '--write', action='store_true', dest='write', default=False,
         help='Write the updates in the source file')
-    parser.add_argument(
+    file_group.add_argument(
         '--indent', dest='indentation', type=int, default=32,
         help='Spaces used when indenting "key = value" (default: 32)')
-    parser.add_argument(
+    file_group.add_argument(
         '--sorting', dest='sorting', default='', choices=['alpha', 'length'],
         help='Sorting algorithm used on the keys when writing source file '
         '(default: None)')
-    parser.add_argument(
+    network_group = parser.add_argument_group('Network')
+    network_group.add_argument(
         '--service-url',  dest='service_url',
         default='http://pypi.python.org/pypi',
         help='The service to use for checking the packages '
         '(default: http://pypi.python.org/pypi)')
-    parser.add_argument(
+    network_group.add_argument(
         '--timeout', dest='timeout', type=int, default=10,
         help='Timeout for each request (default: 10s)')
-    parser.add_argument(
+    network_group.add_argument(
         '-t', '--threads', dest='threads', type=int, default=10,
         help='Threads used for checking the versions in parallel')
-    parser.add_argument(
+    verbosity_group = parser.add_argument_group('Verbosity')
+    verbosity_group.add_argument(
         '-v', action='count', dest='verbosity', default=1,
         help='Increase verbosity (specify multiple times for more)')
-    parser.add_argument(
+    verbosity_group.add_argument(
         '-q', action='count', dest='quietly', default=0,
         help='Decrease verbosity (specify multiple times for more)')
 
@@ -70,7 +105,9 @@ def cmdline(argv=sys.argv[1:]):
     source = options.source
     try:
         checker = VersionsChecker(
-            source, options.includes, options.excludes,
+            source,
+            options.specifiers, options.prereleases,
+            options.includes, options.excludes,
             options.service_url, options.timeout,
             options.threads)
     except Exception as e:
