@@ -1,14 +1,17 @@
 """Tests for Buildout version checker"""
 import os
 import sys
+import json
 
 from logging import Handler
 from collections import OrderedDict
 from tempfile import NamedTemporaryFile
 try:
+    from urllib2 import URLError
     from cStringIO import StringIO
 except ImportError:  # Python 3
     from io import StringIO
+    from urllib.error import URLError
 
 from unittest import TestCase
 from unittest import TestSuite
@@ -46,56 +49,54 @@ class LazyUnusedVersionsChecker(UnusedVersionsChecker):
             setattr(self, key, value)
 
 
-class PypiServerProxy(object):
+class URLOpener(object):
     """
-    Fake Pypi proxy server.
+    Fake urlopen.
     """
     results = {
-        'egg': [
-            {'name': 'Egg', 'version': '0.2'},
-            {'name': 'EGG', 'version': '0.3'},
-            {'name': 'eggtractor', 'version': '0.42'}
-        ],
-        'error-egg': [{}],
+        'egg': {
+            'releases': ['0.3', '0.2']
+        },
+        'egg-dev': {
+            'releases': ['1.0', '1.1b1']
+        },
+        'error-egg': [],
     }
 
-    def __init__(*ka, **kw):
-        pass
-
-    def search(self, query_dict):
+    def __call__(self, url):
+        package = url.split('/')[-2]
         try:
-            return self.results[query_dict['name']]
+            return StringIO(json.dumps(self.results[package]))
         except KeyError:
-            pass
-        return []
+            raise URLError('404')
 
 
-class StubbedServerProxyTestCase(TestCase):
+class StubbedURLOpenTestCase(TestCase):
     """
-    TestCase enabling a stub around the ServerProxy
-    class used by VersionsChecker.
+    TestCase enabling a stub around the urllib2.urlopen
+    used by VersionsChecker.
     """
 
     def setUp(self):
-        self.stub_server_proxy()
-        super(StubbedServerProxyTestCase, self).setUp()
+        self.stub_url_open()
+        super(StubbedURLOpenTestCase, self).setUp()
 
     def tearDown(self):
-        self.unstub_server_proxy()
-        super(StubbedServerProxyTestCase, self).tearDown()
+        self.unstub_url_open()
+        super(StubbedURLOpenTestCase, self).tearDown()
 
-    def stub_server_proxy(self):
+    def stub_url_open(self):
         """
-        Replace the ServerProxy class used in bvc.
+        Replace the urlopen used in bvc.
         """
-        self.original_server_proxy = checker.ServerProxy
-        checker.ServerProxy = PypiServerProxy
+        self.original_url_open = checker.urlopen
+        checker.urlopen = URLOpener()
 
-    def unstub_server_proxy(self):
+    def unstub_url_open(self):
         """
-        Restaure the original ServerProxy class.
+        Restaure the original urlopen function.
         """
-        checker.ServerProxy = self.original_server_proxy
+        checker.urlopen = self.original_url_open
 
 
 class StubbedListDirTestCase(TestCase):
@@ -177,23 +178,29 @@ class StdOutTestCase(TestCase):
     def setUp(self):
         self.output = StringIO()
         self.saved_stdout = sys.stdout
+        self.saved_stderr = sys.stderr
         sys.stdout = self.output
+        sys.stderr = self.output
         super(StdOutTestCase, self).setUp()
 
     def tearDown(self):
         sys.stdout = self.saved_stdout
-        #self.output.close()
+        sys.stderr = self.saved_stderr
         super(StdOutTestCase, self).tearDown()
 
     def assertStdOut(self, output):
         self.assertEquals(self.output.getvalue(),
                           output)
 
+    def assertInStdOut(self, output):
+        self.assertTrue(output in self.output.getvalue())
 
-class VersionsCheckerTestCase(StubbedServerProxyTestCase):
+
+class VersionsCheckerTestCase(StubbedURLOpenTestCase):
 
     def setUp(self):
-        self.checker = LazyVersionsChecker()
+        self.checker = LazyVersionsChecker(
+            service_url='http://custom.pypi.org/pypi')
         super(VersionsCheckerTestCase, self).setUp()
 
     def test_parse_versions(self):
@@ -245,26 +252,70 @@ class VersionsCheckerTestCase(StubbedServerProxyTestCase):
                 excludes=['Django', 'egg']),
             results)
 
+    def test_build_specifiers(self):
+        self.assertEquals(
+            self.checker.build_specifiers(
+                ('Django', 'zc.buildout'),
+                {'django': '<=1.8',
+                 'extra': '!=1.2'}),
+            [('Django', '<=1.8'), ('zc.buildout', '')])
+
     def test_fetch_last_versions(self):
         self.assertEquals(
             self.checker.fetch_last_versions(
-                ['egg', 'UnknowEgg'], 'service_url', 1, 1),
+                [('egg', ''), ('UnknowEgg', '')], False,
+                'service_url', 1, 1),
             [('egg', '0.3'), ('UnknowEgg', '0.0.0')])
+        self.assertEquals(
+            self.checker.fetch_last_versions(
+                [('egg', '<=0.2'), ('UnknowEgg', '>1.0')], False,
+                'service_url', 1, 1),
+            [('egg', '0.2'), ('UnknowEgg', '0.0.0')])
         results = self.checker.fetch_last_versions(
-            ['egg', 'UnknowEgg'], 'service_url', 1, 2)
+            [('egg', ''), ('UnknowEgg', '')], False,
+            'service_url', 1, 2)
         self.assertEquals(
             dict(results),
             dict([('egg', '0.3'), ('UnknowEgg', '0.0.0')]))
 
     def test_fetch_last_version(self):
         self.assertEquals(
-            self.checker.fetch_last_version('UnknowEgg', 'service_url', 1),
+            self.checker.fetch_last_version(
+                ('UnknowEgg', ''), False, 'service_url', 1),
             ('UnknowEgg', '0.0.0')
         )
         self.assertEquals(
-            self.checker.fetch_last_version('egg', 'service_url', 1),
+            self.checker.fetch_last_version(
+                ('egg', ''), False, 'service_url', 1),
             ('egg', '0.3')
         )
+        self.assertEquals(
+            self.checker.fetch_last_version(
+                ('egg', '<0.3'), False, 'service_url', 1),
+            ('egg', '0.2')
+        )
+
+    def test_fetch_last_version_with_prereleases(self):
+        self.assertEquals(
+            self.checker.fetch_last_version(
+                ('egg-dev', ''), False, 'service_url', 1),
+            ('egg-dev', '1.0')
+        )
+        self.assertEquals(
+            self.checker.fetch_last_version(
+                ('egg-dev', ''), True, 'service_url', 1),
+            ('egg-dev', '1.1b1')
+        )
+        self.assertEquals(
+            self.checker.fetch_last_version(
+                ('egg-dev', '<1.1'), True, 'service_url', 1),
+            ('egg-dev', '1.0')
+        )
+        self.assertEquals(
+            self.checker.fetch_last_version(
+                ('egg-dev', '<=1.1'), True, 'service_url', 1),
+            ('egg-dev', '1.1b1')
+        )
 
     def test_find_updates(self):
         versions = OrderedDict([('egg', '1.5.1'), ('Egg', '0.0.0')])
@@ -567,7 +618,7 @@ class FindUnusedVersionsTestCase(LogsTestCase,
                           'Unused-egg=1.0\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s' % config_file.name)
+            find_unused_versions.cmdline(config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             info=['- 2 versions found in %s.' % config_file.name,
@@ -582,7 +633,7 @@ class FindUnusedVersionsTestCase(LogsTestCase,
                           'Unused-egg=1.0\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s -w' % config_file.name)
+            find_unused_versions.cmdline('%s -w' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             info=['- 2 versions found in %s.' % config_file.name,
@@ -609,7 +660,7 @@ class FindUnusedVersionsTestCase(LogsTestCase,
                           'Unused-egg=1.0\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s -e unused-egg' %
+            find_unused_versions.cmdline('%s -e unused-egg' %
                                          config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
@@ -624,7 +675,7 @@ class FindUnusedVersionsTestCase(LogsTestCase,
                           'Unused-egg=1.0\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s -vvv' % config_file.name)
+            find_unused_versions.cmdline('%s -vvv' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             info=['- 2 versions found in %s.' % config_file.name,
@@ -642,11 +693,11 @@ class FindUnusedVersionsTestCase(LogsTestCase,
                           'Unused-egg=1.0\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s -q' % config_file.name)
+            find_unused_versions.cmdline('%s -q' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertStdOut('')
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s invalid -qqqq')
+            find_unused_versions.cmdline('invalid -qqqq')
         self.assertEqual(context.exception.code, 0)
         self.assertStdOut('')
         config_file.close()
@@ -657,7 +708,7 @@ class FindUnusedVersionsTestCase(LogsTestCase,
         config_file = NamedTemporaryFile()
         config_file.write('[versions]\n'.encode('utf-8'))
         with self.assertRaises(SystemExit) as context:
-            find_unused_versions.cmdline('-s %s' % config_file.name)
+            find_unused_versions.cmdline('%s' % config_file.name)
         self.assertEqual(context.exception.code,
                          "'int' object is not iterable")
         self.listdir_content = original_listdir_content
@@ -671,7 +722,7 @@ class IndentCommandLineTestCase(LogsTestCase,
         config_file.write('[sections]\nKey=Value\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            indent_buildout.cmdline('-s %s' % config_file.name)
+            indent_buildout.cmdline('%s' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             warning=['- %s (re)indented at 32 spaces.' % config_file.name])
@@ -685,7 +736,7 @@ class IndentCommandLineTestCase(LogsTestCase,
 
     def test_invalid_source(self):
         with self.assertRaises(SystemExit) as context:
-            indent_buildout.cmdline('-s invalid.cfg')
+            indent_buildout.cmdline('invalid.cfg')
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(warning=['- invalid.cfg cannot be read.'])
         self.assertStdOut('- invalid.cfg cannot be read.\n')
@@ -695,7 +746,7 @@ class IndentCommandLineTestCase(LogsTestCase,
         config_file.write('[sections]\nKey=Value\n'.encode('utf-8'))
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
-            indent_buildout.cmdline('-s %s -s invalid.cfg' % config_file.name)
+            indent_buildout.cmdline('%s invalid.cfg' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             warning=['- %s (re)indented at 32 spaces.' % config_file.name,
@@ -715,18 +766,18 @@ class IndentCommandLineTestCase(LogsTestCase,
 
     def test_output_none(self):
         with self.assertRaises(SystemExit) as context:
-            indent_buildout.cmdline('-s invalid.cfg -q')
+            indent_buildout.cmdline('invalid.cfg -q')
         self.assertEqual(context.exception.code, 0)
         self.assertStdOut('')
         with self.assertRaises(SystemExit) as context:
-            indent_buildout.cmdline('-s source.cfg -qqqqqqq')
+            indent_buildout.cmdline('source.cfg -qqqqqqq')
         self.assertEqual(context.exception.code, 0)
         self.assertStdOut('')
 
 
 class CheckUpdatesCommandLineTestCase(LogsTestCase,
                                       StdOutTestCase,
-                                      StubbedServerProxyTestCase):
+                                      StubbedURLOpenTestCase):
 
     def test_no_args_no_source(self):
         with self.assertRaises(SystemExit) as context:
@@ -762,6 +813,7 @@ class CheckUpdatesCommandLineTestCase(LogsTestCase,
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             debug=["'versions' section not found in versions.cfg.",
+                   '!> http://pypi.python.org/pypi/unavailable/json 404',
                    '-> Last version of unavailable is 0.0.0.'],
             info=['- 1 packages need to be checked for updates.',
                   '> Fetching latest datas for unavailable...',
@@ -782,7 +834,7 @@ class CheckUpdatesCommandLineTestCase(LogsTestCase,
         config_file = NamedTemporaryFile()
         with self.assertRaises(SystemExit) as context:
             check_buildout_updates.cmdline(
-                '-i egg -w -s %s' % config_file.name)
+                '-i egg -w %s' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         config_file.seek(0)
         self.assertEquals(
@@ -799,7 +851,7 @@ class CheckUpdatesCommandLineTestCase(LogsTestCase,
         config_file.seek(0)
         with self.assertRaises(SystemExit) as context:
             check_buildout_updates.cmdline(
-                '-e excluded -w -s %s' % config_file.name)
+                '-e excluded -w %s' % config_file.name)
         self.assertEqual(context.exception.code, 0)
         self.assertLogs(
             debug=['-> Last version of egg is 0.3.',
@@ -876,10 +928,39 @@ class CheckUpdatesCommandLineTestCase(LogsTestCase,
             "[versions]\n"
             "egg                             = 0.3\n")
 
+    def test_output_max_specifiers(self):
+        with self.assertRaises(SystemExit) as context:
+            check_buildout_updates.cmdline('-i egg -s egg:<0.3 -vvv')
+        self.assertEqual(context.exception.code, 0)
+        self.assertStdOut(
+            "'versions' section not found in versions.cfg.\n"
+            "- 1 packages need to be checked for updates.\n"
+            "> Fetching latest datas for egg...\n"
+            "-> Last version of egg<0.3 is 0.2.\n"
+            "=> egg current version (0.0.0) and "
+            "last version (0.2) are different.\n"
+            "- 1 package updates found.\n"
+            "[versions]\n"
+            "egg                             = 0.2\n")
+
+    def test_specifiers_errors(self):
+        with self.assertRaises(SystemExit) as context:
+            check_buildout_updates.cmdline('-i egg -s egg<0.3')
+        self.assertEqual(context.exception.code, 2)
+        self.assertInStdOut('error: argument -s/--specifier: '
+                            'key:value syntax not followed')
+
+        with self.assertRaises(SystemExit) as context:
+            check_buildout_updates.cmdline('-i egg -s egg:')
+        self.assertEqual(context.exception.code, 2)
+        self.assertInStdOut('error: argument -s/--specifier: '
+                            'key or value are empty')
+
     def test_handle_error(self):
         with self.assertRaises(SystemExit) as context:
             check_buildout_updates.cmdline('-i error-egg')
-        self.assertEqual(context.exception.code, "'name'")
+        self.assertEqual(context.exception.code,
+                         "list indices must be integers, not str")
 
 
 loader = TestLoader()
